@@ -57,6 +57,10 @@ Step.prototype.draw = function (surface) {
   throw new Error("Draw: method not implemened.");
 };
 
+// reset run state
+Step.prototype.resetState = function () {
+}; 
+  
 Step.prototype.run = function (task) {
   throw new Error("Run: method not implemened.");
 };
