@@ -43,27 +43,30 @@ static void SetProxyHTTP(MozApp *mozApp, CF_Logger *logger,
 
     rv = mozApp->SetCharPref("network.proxy.http", host);
     if (NS_FAILED(rv))
-        logger->printf("engine", "network.proxy.http failed host=%s "
-                       "rv=%08x", host, rv);
+        logger->printf("engine", YLOG_WARN, "network.proxy.http failed "
+                       "host=%s rv=%08x", host, rv);
 
     rv = mozApp->SetIntPref("network.proxy.http_port", port);
     if (NS_FAILED(rv))
-        logger->printf("engine", "network.proxy.http_port failed port=%d "
-                       "rv=%08x", port, rv);
+        logger->printf("engine", YLOG_WARN, 
+                       "network.proxy.http_port failed port=%d rv=%08x",
+                       port, rv);
 
     rv = mozApp->SetCharPref("network.proxy.ssl", host);
     if (NS_FAILED(rv))
-        logger->printf("engine", "network.proxy.ssl failed host=%s "
-                       "rv=%08x", host, rv);
+        logger->printf("engine", YLOG_WARN,
+                       "network.proxy.ssl failed host=%s rv=%08x", host, rv);
 
     rv = mozApp->SetIntPref("network.proxy.ssl_port", port);
     if (NS_FAILED(rv))
-        logger->printf("engine", "network.proxy.ssl_port failed port=%d "
-                       "rv=%08x", port, rv);
+        logger->printf("engine", YLOG_WARN,
+                       "network.proxy.ssl_port failed port=%d rv=%08x",
+                       port, rv);
 
     rv = mozApp->SetIntPref("network.proxy.type", type);
     if (NS_FAILED(rv))
-        logger->printf("engine", "network.proxy.type 1 failed type=%d "
+        logger->printf("engine", YLOG_WARN,
+                       "network.proxy.type 1 failed type=%d "
                        " rv=%08x", type, rv);
 }
 
@@ -150,7 +153,8 @@ gboolean child_input_handler(GIOChannel *source,
         gtk_main_quit();
     else
     {
-        get_CF_Embed()->logger->printf("engine", "unexpected event=%d",
+        get_CF_Embed()->logger->printf("engine", YLOG_WARN,
+                                       "unexpected event=%d",
                                        (int) condition);
         gtk_main_quit();
     }
@@ -162,7 +166,7 @@ void CF_Embed::main()
 {
     const char *str = display.set_display();
     if (str)
-        logger->printf("engine", "DISPLAY=%s", str);
+        logger->printf("engine", YLOG_LOG, "DISPLAY=%s", str);
 
     global_cf_embed = this;
 
@@ -224,7 +228,7 @@ void CF_Embed::main()
                    child_input_handler, 0);
 
     yaz_timing_stop(task_timing);
-    logger->printf("timing", "boot user=%f real=%f sys=%f",
+    logger->printf("timing", YLOG_LOG, "boot user=%f real=%f sys=%f",
             yaz_timing_get_user(task_timing),
             yaz_timing_get_real(task_timing),
             yaz_timing_get_sys(task_timing));
@@ -268,7 +272,7 @@ void CF_Embed::result_handler(bool success, const char *str)
     write_response(wrbuf_buf(w), wrbuf_len(w));
     wrbuf_destroy(w);
     yaz_timing_stop(task_timing);
-    logger->printf("timing", "%s user=%f real=%f sys=%f",
+    logger->printf("timing", YLOG_LOG, "%s user=%f real=%f sys=%f",
                    wrbuf_cstr(current_task),
                    yaz_timing_get_user(task_timing),
                    yaz_timing_get_real(task_timing),
@@ -301,7 +305,8 @@ void CF_Embed::write_response(const void *buf, size_t count)
     size_t r = write(response_fd[1], buf, count);
     if (r != count)
     {
-        logger->printf("engine", "write_response returned %lld, expected %lld",
+        logger->printf("engine", YLOG_WARN,
+                       "write_response returned %lld, expected %lld",
                        (long long) r, (long long) count);
     }
 }
@@ -318,7 +323,8 @@ void CF_Embed::input_handler(GIOChannel *source)
             g_io_channel_read_line(source, &str_return, &length, 0, &gerror);
         if (status == G_IO_STATUS_ERROR)
         {
-            logger->printf("engine", "g_io_channel_read_line failed: %s",
+            logger->printf("engine", YLOG_WARN,
+                           "g_io_channel_read_line failed: %s",
                            gerror->message);
             WRBUF w = wrbuf_alloc();
             wrbuf_printf(w, "FAIL g_io_channel_read_line failed: %s\n",
@@ -420,7 +426,8 @@ void CF_Embed::input_handler(GIOChannel *source)
             }
             else if (!strcmp(cmd, "dom_string"))
             {
-                logger->printf("engine", "dom_string command handler.\n");
+                logger->printf("engine", YLOG_DEBUG,
+                               "dom_string command handler.\n");
                 char *dom_str = 0;
 		if (connector->DOMString(&dom_str))
                 {
@@ -459,11 +466,11 @@ CF_Engine::CF_Engine(enum Flags flags,
                      const char *app_path,
                      const char *profile_path, const char *proxy,
                      const char *loglevel,
-                     CF_Logger *logger, int fd)
+                     CF_Logger *logger, int fd, int capt_stdout)
 {
     m_priv = 0;
     init(flags, 0, 0, builder_path, app_path, profile_path, proxy,
-         loglevel, logger, fd);
+         loglevel, logger, fd, capt_stdout);
 }
 
 bool CF_Embed::wait_init_handshake(std::string &error_msg)
@@ -476,7 +483,8 @@ bool CF_Embed::wait_init_handshake(std::string &error_msg)
     bool res = wait_result_parent(&output, 20000, &sys_err);
     yaz_timing_stop(myt);
 
-    logger->printf("timing", "handshake user=%f real=%f sys=%f",
+    logger->printf("timing", YLOG_LOG,
+                   "handshake user=%f real=%f sys=%f",
                    yaz_timing_get_user(myt),
                    yaz_timing_get_real(myt),
                    yaz_timing_get_sys(myt));
@@ -486,7 +494,8 @@ bool CF_Embed::wait_init_handshake(std::string &error_msg)
     {
         if (output && *output)
         {
-            logger->printf("engine", "engine create fail: %s", output);
+            logger->printf("engine", YLOG_LOG,
+                           "engine create fail: %s", output);
             error_msg = output;
         }
         else
@@ -500,7 +509,7 @@ bool CF_Embed::wait_init_handshake(std::string &error_msg)
     return true;
 }
 
-bool CF_Embed::spawn_fork(std::string &error_msg, int fd)
+bool CF_Embed::spawn_fork(std::string &error_msg, int fd, int capt_stdout)
 {
     int i, lfd = -1;
     FILE *yaz_log_FILE = yaz_log_file();
@@ -524,11 +533,14 @@ bool CF_Embed::spawn_fork(std::string &error_msg, int fd)
         if (fd != -1)
         {
             close(0);
-            dup(fd);
+            if (capt_stdout)
+                dup(fd);
             close(1);
-            dup(fd);
+            if (capt_stdout)
+                dup(fd);
             close(2);
-            dup(fd);
+            if (capt_stdout)
+                dup(fd);
         }
         main();
         exit(0);
@@ -543,7 +555,8 @@ bool CF_Embed::spawn_fork(std::string &error_msg, int fd)
         close(request_fd[0]);
         request_fd[0] = -1;
 
-        logger->printf("engine", "CF fork: parentpid=%lld childpid=%lld",
+        logger->printf("engine", YLOG_LOG,
+                       "CF fork: parentpid=%lld childpid=%lld",
                        (long long) getpid(), (long long) pid);
 
         return wait_init_handshake(error_msg);
@@ -569,7 +582,7 @@ void CF_Engine::init(enum Flags flags,
                      int argc, char **argv, const char *builder_path,
                      const char *app_path, const char *profile_path,
                      const char *proxy, const char *loglevel,
-                     CF_Logger *logger, int fd)
+                     CF_Logger *logger, int fd, int capt_stdout)
 {
     if (!builder_path)
 	throw CF_Engine_Error_System("builder_path NULL");
@@ -578,7 +591,7 @@ void CF_Engine::init(enum Flags flags,
                                    proxy, loglevel, logger,
                                    fd);
 
-    embed->logger->printf("engine",
+    embed->logger->printf("engine", YLOG_LOG,
                           "init: flags=%d, builder_path=%s app_path=%s "
                           "profile_path=%s proxy=%s", flags,
                           builder_path ? builder_path : "null",
@@ -602,7 +615,7 @@ void CF_Engine::init(enum Flags flags,
     bool ret = false;
     if ((flags & 1) == FORK)
     {
-        ret = embed->spawn_fork(msg, fd); 
+        ret = embed->spawn_fork(msg, fd, capt_stdout);
     }
     else
     {
@@ -631,17 +644,14 @@ CF_Engine::~CF_Engine()
     delete m_priv;
 }
 
-void CF_Embed::printf(const char *level, const char *fmt, ...)
+void CF_Embed::printf(const char *module, int level, const char *fmt, ...)
 {
     va_list ap;
     char buf[4096];
 
     va_start(ap, fmt);
-    if (strncmp(level, "RT", 2))
-    {
-        yaz_vsnprintf(buf, sizeof(buf)-1, fmt, ap);
-        ::printf("%s %s\n", level, buf);
-    }
+    yaz_vsnprintf(buf, sizeof(buf)-1, fmt, ap);
+    ::printf("%s %s\n", module, buf);
     va_end(ap);
 }
 
@@ -718,7 +728,7 @@ CF_Embed::CF_Embed(int a_argc, char **a_argv,
     {
         const char *e = display.get_error();
 
-        logger->printf("engine", "display lock: %s", e);
+        logger->printf("engine", YLOG_FATAL, "display lock: %s", e);
 
         throw CF_Engine_Error_System(e);
     }
@@ -784,7 +794,7 @@ CF_Embed::~CF_Embed()
         bool sys_err = false;
         wait_result_parent(&output, 20000, &sys_err);
 
-        logger->printf("engine", "waiting for threads to join");
+        logger->printf("engine", YLOG_LOG, "waiting for threads to join");
         pthread_join(child_thread, 0); // and wait for it to stop
     }
     if (request_fd[0] != -1)
@@ -800,10 +810,11 @@ CF_Embed::~CF_Embed()
     mozApp = 0;
     if (pid)
     {
-        logger->printf("engine", "kill %lld", (long long) pid);
+        logger->printf("engine", YLOG_LOG, "kill %lld", (long long) pid);
         int r = kill(pid, SIGTERM);
         if (r == -1)
-            logger->printf("engine", "kill %lld failed", (long long) pid);
+            logger->printf("engine", YLOG_WARN,
+                           "kill %lld failed", (long long) pid);
         else
         {
             int status;
@@ -818,7 +829,7 @@ CF_Embed::~CF_Embed()
             WRBUF w = wrbuf_alloc();
             wrbuf_printf(w, "/bin/rm -r %s", profile_path);
             system(wrbuf_cstr(w));
-            logger->printf("engine", "remove %s", profile_path);
+            logger->printf("engine", YLOG_LOG, "remove %s", profile_path);
             wrbuf_destroy(w);
         }
     }
@@ -850,7 +861,8 @@ bool CF_Embed::wait_result_parent(const char **output, int timeout_ms, bool *sys
             r = poll(&fds, 1, 10000);
             if (r == 0)
             {
-                logger->printf("engine", "poll waited for 10 secs. pid=%lld round=%d",
+                logger->printf("engine", YLOG_LOG,
+                               "poll waited for 10 secs. pid=%lld round=%d",
                                (long long) pid, round);
             }
             timeout_ms = timeout_ms - 10000;
@@ -874,7 +886,7 @@ bool CF_Embed::wait_result_parent(const char **output, int timeout_ms, bool *sys
         else if (r == -1)
         {
             int x = errno;
-            logger->printf("engine", "poll: %s", strerror(x));
+            logger->printf("engine", YLOG_LOG, "poll: %s", strerror(x));
             if (x == EINTR)
                 continue;
             wrbuf_printf(result_wrbuf, "FAIL poll: %s", strerror(x));
@@ -924,7 +936,8 @@ void CF_Embed::get_result_parent(const char *default_msg, const char **output)
         {
             if (sys_err)
             {
-    	        logger->printf("engine", "sys err for %s", default_msg);
+    	        logger->printf("engine", YLOG_WARN, 
+                               "sys err for %s", default_msg);
                 throw CF_Engine_Error_System(*output);
             }
 	    else
@@ -1160,6 +1173,18 @@ const char *CF_Engine_Error::what()
     return msg;
 }
 
+const char *CF_Logger::logstr(int level)
+{
+    if (level & YLOG_FATAL)
+        return "fatal";
+    if (level & YLOG_DEBUG)
+        return "debug";
+    if (level & YLOG_WARN)
+        return "warn";
+    if (level & YLOG_LOG)
+        return "log";
+    return "-";
+}
 
 /*
  * Local variables:
