@@ -11,11 +11,14 @@
 #ifndef cf_logger_h
 #define cf_logger_h 1
 
+#include <yaz/log.h>
+
 class CF_Logger {
 public:
+    static const char *logstr(int level);
     CF_Logger(){};
     virtual ~CF_Logger(){};
-    virtual void printf(const char *level, const char *fmt, ...) = 0;
+    virtual void printf(const char *module, int level, const char *fmt, ...) = 0;
 };
 
 #endif
