@@ -30,6 +30,8 @@
 #include <yaz/diagbib1.h>
 #include <yaz/oid_db.h>
 #include <yaz/snprintf.h>
+#include <sys/uio.h>
+#include <fcntl.h>
 
 #include "cf_assert.h"
 #include "cf_factory.h"
@@ -70,14 +72,19 @@ namespace metaproxy_1 {
             std::string session_name;
             bool in_use;
             bool destroyed;
-            FILE *log_file;
+            int log_file;
+            std::string log_modules;
             mp::wrbuf file_area;
             Session(std::string session_name);
             ~Session();
             void log(int level, const char *fmt, ...);
             CF_Metadata metadata;
+            bool module_check(const char *actual);
+
+            boost::mutex log_mutex;
+            mp::wrbuf log_wr;
         public:
-            void printf(const char *level, const char *fmt, ...);
+            void printf(const char *module, int level, const char *fmt, ...);
         };
 
         class CF::ZSession {
@@ -430,7 +437,28 @@ bool yf::CF::ZSession::get_record_xml(int n, WRBUF wrb, int *errcode,
 }
 
 
-void yf::CF::Session::printf(const char *level, const char *fmt, ...)
+bool yf::CF::Session::module_check(const char *actual)
+{
+    const char *cp = log_modules.c_str();
+    if (!*cp)
+        return true;
+    for (;;)
+    {
+        const char *cp1 = strchr(cp, ',');
+        if (!cp1)
+            break;
+
+        if (!strncmp(cp, actual, cp1 - cp))
+            return true;
+        cp = cp1 + 1;
+    }
+    if (!strcmp(cp, actual))
+        return true;
+    return false;
+}
+
+void yf::CF::Session::printf(const char *module, int level,
+                             const char *fmt, ...)
 {
     va_list ap;
     char buf[4096];
@@ -441,11 +469,15 @@ void yf::CF::Session::printf(const char *level, const char *fmt, ...)
     const char *lead = "[JavaScript Warning: ";
     if (strncmp(lead, buf, strlen(lead)))
     {
-        yaz_log(YLOG_LOG, "%s %s %s", session_name.c_str(), level, buf);
-        if (log_file)
+        yaz_log(level, "%s %s %s", session_name.c_str(), module, buf);
+        if (log_file != -1 && module_check(module))
         {
-            fprintf(log_file, "%s %s\n", level, buf);
-            fflush(log_file);
+            boost::mutex::scoped_lock lock(log_mutex);
+
+            wrbuf_rewind(log_wr);
+            wrbuf_printf(log_wr, "%s %s %s\n", CF_Logger::logstr(level),
+                         module, buf);
+            write(log_file, log_wr.buf(), log_wr.len());
         }
     }
     va_end(ap);
@@ -453,15 +485,22 @@ void yf::CF::Session::printf(const char *level, const char *fmt, ...)
 
 void yf::CF::Session::log(int level, const char *fmt, ...)
 {
+    const char *module = "engine";
     va_list ap;
     char buf[4096];
 
     va_start(ap, fmt);
     yaz_vsnprintf(buf, sizeof(buf)-1, fmt, ap);
     yaz_log(level, "%s %s", session_name.c_str(), buf);
-    if (log_file)
-        fprintf(log_file, "%s\n", buf);
+    if (log_file != -1 && module_check(module))
+    {
+        boost::mutex::scoped_lock lock(log_mutex);
 
+        wrbuf_rewind(log_wr);
+        wrbuf_printf(log_wr, "%s %s %s\n", CF_Logger::logstr(level),
+                     module, buf);
+        write(log_file, log_wr.buf(), log_wr.len());
+    }
     va_end(ap);
 }
 
@@ -469,7 +508,7 @@ yf::CF::Session::Session(std::string a_session_name)
     : session_name(a_session_name)
 {
     this->engine = 0;
-    this->log_file = 0;
+    this->log_file = -1;
     this->in_use = true;
     this->destroyed = false;
     this->age = 0;
@@ -478,8 +517,8 @@ yf::CF::Session::Session(std::string a_session_name)
 yf::CF::Session::~Session()
 {
     delete engine;
-    if (log_file)
-        fclose(log_file);
+    if (log_file != -1)
+        close(log_file);
 }
 
 yf::CF::Rep::Rep()
@@ -568,9 +607,9 @@ CF_Engine *yf::CF::Rep::create_connector(CF_Engine::Flags engine_flag,
 
     mp::wrbuf cf_log_fname;
     wrbuf_printf(cf_log_fname, "%s.log", wrbuf_cstr(ses->file_area));
-    FILE *log_file = fopen(wrbuf_cstr(cf_log_fname), "w+");
+    int log_file = open(wrbuf_cstr(cf_log_fname), O_TRUNC|O_CREAT|O_RDWR, 0666);
     ses->log_file = log_file;
-    if (!log_file)
+    if (log_file == -1)
     {
         addinfo = "could not open ";
         addinfo += wrbuf_cstr(cf_log_fname);
@@ -581,8 +620,8 @@ CF_Engine *yf::CF::Rep::create_connector(CF_Engine::Flags engine_flag,
 
     CF_Engine *cf = 0;
     try {
-        cf = cf_factory->create(engine_flag, proxy, loglevel,
-                                ses, fileno(log_file));
+        cf = cf_factory->create(engine_flag, proxy, loglevel, ses, log_file,
+                                ses->module_check("stdout") ? 1 : 0);
     }
     catch (CF_Engine_Error &e)
     {
@@ -665,12 +704,15 @@ void yf::CF::Rep::post_connector(Package &package,
         return;
     }
     int i;
+    std::string logmodules;
     for (i = 0; i < no_parms; i++)
     {
         if (!strcmp(names[i], "proxy"))
             proxy = values[i];
         else if (!strcmp(names[i], "loglevel"))
             loglevel = values[i];
+        else if (!strcmp(names[i], "logmodules"))
+            logmodules = values[i];
         else if (!strcmp(names[i], "thread"))
         {
             if (values[i] && *values[i] == '1')
@@ -694,7 +736,7 @@ void yf::CF::Rep::post_connector(Package &package,
 
         sprintf(session_name_str, "%d", id);
         ses = new Session(session_name_str);
-
+        ses->log_modules = logmodules;
     }
 
     std::string addinfo;
@@ -1541,20 +1583,18 @@ void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
             mp::odr odr;
             Z_GDU *zgdu_res
                 = odr.create_HTTP_Response(package.session(), req, 200);
-            cf_http_msg(zgdu_res, odr, "only POST allowed for without", 405);
+            cf_http_msg(zgdu_res, odr, "only POST allowed for without /", 405);
             package.response() = zgdu_res;
         }
     }
     else
     {    // POST /connector/id/args or DELETE /connector/id
         mp::odr odr;
-        char args[60];
-        int id = 0;
-
-        *args = '\0';
-        int no_args = sscanf(suffix_path, "/%d/%59s", &id, args);
-
-        if (no_args < 1)
+        char **names = 0;
+        char **values = 0;
+        int no_parms = 0;
+        const char *cp1 = suffix_path + 1;
+        if (!*cp1)
         {
             Z_GDU *zgdu_res
                 = odr.create_HTTP_Response(package.session(), req, 200);
@@ -1563,6 +1603,23 @@ void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
             package.response() = zgdu_res;
             return ;
         }
+        int id = atoi(cp1);
+        cp1 = strchr(cp1, '/');
+        char cmd[60];
+        if (cp1)
+        {
+            size_t i = 0;
+            while (*++cp1 && *cp1 != '?')
+            {
+                if (i < sizeof(cmd) - 1)
+                    cmd[i++] = *cp1;
+            }
+            cmd[i] = 0;
+            if (*cp1 == '?')
+                no_parms = yaz_uri_to_array(cp1, odr, &names, &values);
+        }
+        else
+            *cmd = 0;
         if (!strcmp(req->method, "POST"))
         {
             Session *cf_session = 0;
@@ -1583,30 +1640,56 @@ void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
                 }
                 cf_session = map_it->second;
 
-                if (!strcmp(args, "log"))
+                if (!strcmp(cmd, "log"))
                 {
+                    bool clear_log = false;
+                    if (no_parms == 1)
+                    {
+                        if (!strcmp(names[0], "clear"))
+                        {
+                            if (values[0] && values[0][0] == '1')
+                                clear_log = true;
+                        }
+                        else
+                        {
+                            Z_GDU *zgdu_res
+                                = odr.create_HTTP_Response(package.session(), req, 200);
+                            cf_http_msg(zgdu_res, odr, "bad argument for log", 400);
+                            package.response() = zgdu_res;
+                            return ;
+                        }
+                    }
                     Z_GDU *zgdu_res
                         = odr.create_HTTP_Response(package.session(), req, 200);
                     Z_HTTP_Response *response = zgdu_res->u.HTTP_Response;
 
                     z_HTTP_header_add_content_type(odr, &response->headers,
                                                    "text/plain", 0);
-                    if (cf_session->log_file)
+                    if (cf_session->log_file != -1)
                     {
-                        long sz = ftell(cf_session->log_file);
+                        int fd = cf_session->log_file;
+                        mp::wrbuf cf_log_fname;
+                        wrbuf_printf(cf_log_fname,
+                                     "%s.log", wrbuf_cstr(cf_session->file_area));
+
+                        long sz = lseek(fd, 0, SEEK_END);
                         if (sz > 0)
                         {
-                            int fd = fileno(cf_session->log_file);
                             char *buf = (char*) odr_malloc(odr, sz);
                             pread(fd, buf, sz, 0);
                             response->content_buf = buf;
                             response->content_len = sz;
+                            if (clear_log)
+                            {
+                                ftruncate(fd, 0);
+                                lseek(fd, 0L, SEEK_SET);
+                            }
                         }
                     }
                     package.response() = zgdu_res;
                     return;
                 }
-                else if (!strcmp(args, "screen_shot"))
+                else if (!strcmp(cmd, "screen_shot"))
                 {
                     Z_GDU *zgdu_res = 0;
                     WRBUF fname = wrbuf_alloc();
@@ -1667,7 +1750,7 @@ void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
             }
             // args is one of engine methods: "run_task", "run_task_opt",
             // "run_tests", "load_cf", "dom_string"
-            task_connector(package, req, cf_session, args);
+            task_connector(package, req, cf_session, cmd);
             {
                 boost::mutex::scoped_lock lock(m_mutex);
                 cf_session->in_use = false;
